@@ -12,7 +12,6 @@
 #include <time.h>
     
 #include <vdr/interface.h>
-#include <vdr/i18n.h>
 #include <vdr/config.h>
 
 #include "menu.h"
@@ -20,9 +19,6 @@
 #include "setup.h"
 #include "txtrecv.h"
 
-
-#define TXTROOT "/vtx"
-
 #define GET_HUNDREDS(x) ( ( (x) - ((x)%256) ) /256 )
 #define GET_TENS(x)  ( (( (x) - ((x)%16) )%256 ) /16 )
 #define GET_ONES(x)   ( (x)%16 )
@@ -158,8 +154,7 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
                   ChannelSwitched(selectingChannelNumber);
                else {
                   needClearMessage=true;
-                  //string tranlated in VDR's i18n.c
-                  Display::DrawMessage(tr("*** Invalid Channel ***"));
+                  Display::DrawMessage(trVDR("*** Invalid Channel ***"));
                }
             } else {
                ShowPage();
@@ -315,10 +310,7 @@ void TeletextBrowser::ExecuteAction(eTeletextAction e) {
                //suspendedReceiving=(!suspendedReceiving);
                break;*/
             case DarkScreen:
-               if (Display::GetBackgroundColor() == clrBlack)
-                  Display::SetBackgroundColor((tColor)ttSetup.configuredClrBackground);
-               else
-                  Display::SetBackgroundColor(clrBlack);
+               ChangeBackground();
                break;
             default:
                //In osdteletext.c, numbers are thought to be decimal, the setup page
@@ -344,6 +336,27 @@ void TeletextBrowser::ExecuteAction(eTeletextAction e) {
          }
 }
 
+// 3-state toggling between configured->transparent->black.
+// If configured is black or transparent, do 2-state transparent->black only.
+void TeletextBrowser::ChangeBackground()
+{
+   tColor clrConfig = (tColor)ttSetup.configuredClrBackground;
+   tColor clrCurrent = Display::GetBackgroundColor();
+
+   if (clrCurrent == clrConfig)
+      if (clrConfig == clrTransparent)
+         Display::SetBackgroundColor(clrBlack);
+      else
+         Display::SetBackgroundColor(clrTransparent);
+   else if (clrCurrent == clrBlack)
+      if (clrConfig == clrBlack)
+         Display::SetBackgroundColor(clrTransparent);
+      else
+         Display::SetBackgroundColor(clrConfig);
+   else // clrCurrent == clrTransparent
+      Display::SetBackgroundColor(clrBlack);
+}
+
 eTeletextAction TeletextBrowser::TranslateKey(eKeys Key) {
    switch(Key) {
       case kRed:     return (eTeletextAction)ttSetup.mapKeyToAction[ActionKeyRed];
@@ -550,24 +563,14 @@ void TeletextBrowser::ShowPageNumber() {
 
 void TeletextBrowser::ShowAskForChannel() {
    if (selectingChannel) {
-      char *str;
-      if (selectingChannelNumber>0)
-         asprintf(&str,"%s%d", tr("Channel (press OK): "), selectingChannelNumber);
-      else
-         asprintf(&str,"%s", tr("Channel (press OK): ") );
+      cString str = cString::sprintf(selectingChannelNumber > 0 ? "%s%d" : "%s", tr("Channel (press OK): "), selectingChannelNumber);
       Display::DrawMessage(str);
-      free(str);
    }
 }
 
 //this is taken and adapted from the teletext plugin since it uses its data
 bool TeletextBrowser::DecodePage() {
    // Load the page and decodes it
-   #ifdef timingdebug
-   cTime t;
-   t.Start();
-   #endif
-
    unsigned char cache[40*24+12];
    StorageHandle fd;
    // Take a look if there is a xxx-00 page
@@ -594,10 +597,6 @@ bool TeletextBrowser::DecodePage() {
       ShowPageNumber();
       UpdateClock();
       Display::ReleaseFlush();
-
-      #ifdef timingdebug
-      t.Stop("Full Page Update");
-      #endif
    } else {
       // page doesn't exist
       currentSubPage--;
@@ -640,17 +639,6 @@ void TeletextBrowser::UpdateClock() {
       Display::DrawClock();
 }
 
-
-ChannelStatus::ChannelStatus() 
-{
-}
-
-
-void ChannelStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber) {
-   if (Device->IsPrimaryDevice() && ChannelNumber>0) 
-      TeletextBrowser::ChannelSwitched(ChannelNumber);
-}
-
 TeletextSetup ttSetup;
 
 TeletextSetup::TeletextSetup() {
