@@ -52,6 +52,7 @@ protected:
 
 class Storage : public RootDir {
 public:
+   virtual ~Storage();
    enum StorageSystem { StorageSystemLegacy, StorageSystemPacked };
    //must be called before the first call to instance()
    static void setSystem(StorageSystem system);
@@ -83,7 +84,7 @@ protected:
    bool exists(const char* file);
    
    long byteCount;
-   char *currentDir;
+   cString currentDir;
 private:
    static StorageSystem system;
    int storageOption;
@@ -93,6 +94,7 @@ private:
 class LegacyStorage : public Storage {
 public:
    LegacyStorage();
+   virtual ~LegacyStorage();
    virtual void cleanUp();
    
    virtual StorageHandle openForWriting(PageID page);
@@ -172,30 +174,19 @@ protected:
    bool running;
 public:
    cTxtReceiver(int TPid, tChannelID chan);
-   ~cTxtReceiver();
+   virtual ~cTxtReceiver();
+   virtual void Stop();
 };
 
-class cTxtStatus : public cStatus/*, public cThread*/ {
+class cTxtStatus : public cStatus {
 private:
    cTxtReceiver *receiver;
-   //bool running;
-   //cCondVar condVar;
-   //cMutex mutex;
-   //int count;
+   tChannelID currentLiveChannel;
 protected:
-   int TPid;
-   tChannelID chan;
-   //bool doNotSuspend;
-   //bool doNotReceive; 
    virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
-   //virtual void Action();
-   void CheckCreateReceiver();
-   void CheckDeleteReceiver();
 public:
    cTxtStatus(void);
    ~cTxtStatus();
-   //void ForceReceiving(bool onOrOff);
-   //void ForceSuspending(bool onOrOff);
 };
 
 
